﻿using System;
using Unity.Collections;
using Unity.Entities;
using UnityEngine;
using CoreLib;
using Il2CppSystem.Collections.Generic;
using Unity.Transforms;

namespace AutoPlant
{
    public class AutoPlantSystem : MonoBehaviour
    {
        public static AutoPlantSystem instance;
        internal World serverWorld;

        private const int life = 999999;
        private float waitTime;
        private const float refreshTime = 3;
        private PlayerController pc;
        private SaveManager save;
        private int talentIndex = 6;
        private SkillTalentsTable table;

        public AutoPlantSystem(IntPtr ptr) : base(ptr) { }
        public void OnServerStarted(World world)
        {
            serverWorld = world;
        }
        public bool isNullOfserverWorld() => serverWorld == null;
        public void destroyWorld() => serverWorld = null;
        public void initiateTime() => waitTime = refreshTime;
        public void setZero() => waitTime = 0;
        public bool isZero() => waitTime == 0;

        private void Awake()
        {
            instance = this;
        }
        private int R(float f)
        {
            int s = Math.Sign(f);
            f = Math.Abs(f);
            int t = (int)f;
            f -= t;
            f = f < 0.5f ? 0 : 1;
            return s * (t + (int)f);
        }
        private void Update()
        {
            if (serverWorld == null) return;
            waitTime -= Time.deltaTime;

            if (waitTime <= 0)
            {
                EntityQuery query =
                    serverWorld.EntityManager.CreateEntityQuery(
                        ComponentType.ReadOnly<PlantCD>());
                NativeArray<Entity> result = query.ToEntityArray(Allocator.Temp);
                foreach (Entity entity in result)
                {
                    ObjectDataCD objectData = EntityUtility.GetObjectData(entity, serverWorld);
                    if (objectData.variation == 1) continue;
                    HealthCD healthCd = serverWorld.EntityManager.GetComponentData<HealthCD>(entity);
                    GrowingCD growingCD = serverWorld.EntityManager.GetComponentData<GrowingCD>(entity);

                    if (!growingCD.hasReachedFinalStage || healthCd.maxHealth < life)
                    {
                        healthCd.maxHealth = life;
                        healthCd.health = healthCd.maxHealth;
                        serverWorld.EntityManager.SetComponentData(entity, healthCd);
                    }
                    else if (healthCd.health < healthCd.maxHealth && healthCd.health > 0)
                    {
                        Unity.Mathematics.float3 position = serverWorld.EntityManager.GetComponentData<Translation>(entity).Value;
                        if (pc == null) pc = Players.GetCurrentPlayer();
                        if (save == null) save = GameManagers.GetMainManager()._saveManager;
                        position = EntityMonoBehaviour.ToRenderFromWorld(position);
                        List<int> points = save.GetSkillTalentTreesPoints(SkillID.Gardening);
                        int level = points[talentIndex];
                        if (table == null) table = GameManagers.GetMainManager()._modManager.skillTalentsTable;
                        int point = table.GetConditionDataForSkillTalent(SkillID.Gardening, talentIndex, level).value;
                        int varia = 0;
                        healthCd.health = 0;
                        serverWorld.EntityManager.SetComponentData(entity, healthCd);
                        if (objectData.variation == 0)
                        {
                            UnityEngine.Random.seed = Math.Abs(Guid.NewGuid().GetHashCode());
                            int rate = UnityEngine.Random.RandomRange(0, 100);
                            if (rate < point && objectData.objectID != ObjectID.GrubKapokPlant) varia = 1;
                            pc.playerCommandSystem.CreateEntity(objectData.objectID - 1, position, varia);
                        }
                        else if (objectData.variation == 2)
                        {
                            pc.playerCommandSystem.CreateEntity(objectData.objectID - 1, position, 1);
                        }
                    }
                }
                result.Dispose();
                waitTime = refreshTime;
            }
        }
    }
}
